-- Cyrodiil Alert
-- Author: @Tanthul, Dark Moon Guild (Scourge EU)

CA = {}

CA.name = "CyrodiilAlert"
CA.version = "0.2.4"
CA.initialised = false

-- Default settings.
CA.defaults = {
    locx            	= 655,
    locy            	= 150,
    scaling         	= 1.0,
	notifyDelay			= 4,
	chatOutput			= true,
	onlyChat			= false,
	outside				= true,
	showAttack			= true,
	showAttackEnd		= true,
	showOwnerChanged	= true,
	showEmperors		= true,
	showGates			= true,
	showScrolls			= true,
	showClaim			= true,
    textAlign       	= "CENTER",
}

CA_ACE = LibStub("AceTimer-3.0")

function CA.Initialise(eventCode, addOnName)

	-- Initialize self only.
	if (CA.name ~= addOnName) then return end

	-- Load saved variables.
	CA.vars = ZO_SavedVars:NewAccountWide("CA_SavedVariables", 1, nil, CA.defaults)
    RegisterForAssignedCampaignData()
	
	CyrodiilAlert:ClearAnchors();
	CyrodiilAlert:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, CA.vars.locx, CA.vars.locy)

    
    CyrodiilAlertBG:SetAlpha(0)
    
	CyrodiilAlertNotify:ClearAnchors();
	CyrodiilAlertNotify:SetAnchor(TOP, CyrodiilAlert, TOP, 0, 0)
    CyrodiilAlertNotify:SetWidth( 800 )
    CyrodiilAlertNotify:SetHeight(40 )
	
	CyrodiilAlertNotifyTaken:ClearAnchors();
	CyrodiilAlertNotifyTaken:SetAnchor(TOP, CyrodiilAlertNotify, BOTTOM, 0, 0)
    CyrodiilAlertNotifyTaken:SetWidth( 800 )
    CyrodiilAlertNotifyTaken:SetHeight( 40 )
	
	CyrodiilAlertNotifyExtra:ClearAnchors();
	CyrodiilAlertNotifyExtra:SetAnchor(TOP, CyrodiilAlertNotifyTaken, BOTTOM, 0, 0)
    CyrodiilAlertNotifyExtra:SetWidth( 800 )
    CyrodiilAlertNotifyExtra:SetHeight( 40 )
    
    -- Configuration Menu setup.
    CA.CreateConfigMenu()
    
    -- Event registration.    
    EVENT_MANAGER:RegisterForEvent("CA", EVENT_KEEP_UNDER_ATTACK_CHANGED, CA.OnKeepUnderAttackChanged)
	EVENT_MANAGER:RegisterForEvent("CA", EVENT_KEEP_ALLIANCE_OWNER_CHANGED, CA.OnAllianceOwnerChanged)
	EVENT_MANAGER:RegisterForEvent("CA", EVENT_KEEP_GATE_STATE_CHANGED, CA.OnGateChanged)
	EVENT_MANAGER:RegisterForEvent("CA", EVENT_DEPOSE_EMPEROR_NOTIFICATION, CA.OnDeposeEmperor)
	EVENT_MANAGER:RegisterForEvent("CA", EVENT_CORONATE_EMPEROR_NOTIFICATION, CA.OnCoronateEmperor)
	EVENT_MANAGER:RegisterForEvent("CA", EVENT_ARTIFACT_CONTROL_STATE, CA.OnArtifactControlState)
	EVENT_MANAGER:RegisterForEvent("CA", EVENT_GUILD_CLAIM_KEEP_CAMPAIGN_NOTIFICATION, CA.OnClaimKeep)
	EVENT_MANAGER:RegisterForEvent("CA", EVENT_ACTION_LAYER_POPPED, CA.ShowInterface)
    EVENT_MANAGER:RegisterForEvent("CA", EVENT_ACTION_LAYER_PUSHED, CA.HideInterface)

    CA.initialised = true
end


EVENT_MANAGER:RegisterForEvent("CA", EVENT_ADD_ON_LOADED, CA.Initialise)


function CA.HideInterface(eventCode, layerIndex, activeLayerIndex)
	if (activeLayerIndex == 3) then
		CyrodiilAlert:SetHidden(true)
	end
end

function CA.ShowInterface(...)
    CyrodiilAlert:SetHidden(false)
end


function CA.OnAllianceOwnerChanged(eventCode, keepId, battlegroundContext, owningAlliance)
	if ((CA.vars.outside == false) and (not IsInCampaign())) then
		do return end
	end
	
	if (CA.vars.showOwnerchanged == false) then
		do return end
	end

	local keepName = GetKeepName(keepId)
	local allianceName = GetAllianceName(owningAlliance)
	
	if (timerIdTaken) then
		CA_ACE:CancelTimer(timerIdTaken)
	end
	
	if (CA.vars.chatOutput) then
		d("|cB22222" .. allianceName .. "|r |cE5E5E5has captured|r |cB22222" .. keepName .. "|r")
	end
	
	if (not CA.vars.onlyChat) then
		CyrodiilAlertNotifyTaken:SetText("|cD6660C" .. allianceName .. "|r |cE5E5E5has captured|r |cD6660C" .. keepName .. "|r")

		timerIdTaken = CA_ACE:ScheduleTimer("ClearNotifyTaken", CA.vars.notifyDelay)
	end

end

function CA.OnKeepUnderAttackChanged(eventCode, keepId, battlegroundContext, underAttack)
	if ((CA.vars.outside == false) and (not IsInCampaign())) then
		do return end
	end
	
	local keepName = GetKeepName(keepId)
	
	--Efficiently get the number of sieges. The smart way. Looks like code will be stolen again!:/
	local keepAlliance = GetKeepAlliance(keepId, battlegroundContext)
	local defendSiege = GetNumSieges(keepId, battlegroundContext, keepAlliance)
	local allSiege = GetNumSieges(keepId, battlegroundContext, ALLIANCE_ALDMERI_DOMINION)
	allSiege = allSiege + GetNumSieges(keepId, battlegroundContext, ALLIANCE_DAGGERFALL_COVENANT)
	allSiege = allSiege + GetNumSieges(keepId, battlegroundContext, ALLIANCE_EBONHEART_PACT)
	local attackSiege = allSiege - defendSiege
	
	if (timerId) then
		CA_ACE:CancelTimer(timerId)
	end
	
	if (underAttack) and (CA.vars.showAttack) then
		if (CA.vars.chatOutput) then
			d("|cB22222" .. keepName .. "|r |cE5E5E5is under attack! Sieges A:|cB22222".. attackSiege .. "|r |cE5E5E5/D:|c5CA028" .. defendSiege .. "|r")
		end
		
		if (not CA.vars.onlyChat) then
			CyrodiilAlertNotify:SetText("|cB22222" .. keepName .. "|r |cE5E5E5is under attack! Sieges A:|cB22222".. attackSiege .. "|r |cE5E5E5/D:|c5CA028" .. defendSiege .. "|r")

			timerId = CA_ACE:ScheduleTimer("ClearNotify", CA.vars.notifyDelay)
		end
	elseif (not underAttack) and (CA.vars.showAttackEnd) then
		if (CA.vars.chatOutput) then
			d("|cB22222" .. keepName .. "|r |cE5E5E5is no longer under attack!|r")
		end
		
		if (not CA.vars.onlyChat) then
			CyrodiilAlertNotify:SetText("|cD6660C" .. keepName .. "|r |cE5E5E5is no longer under attack!|r")

			timerId = CA_ACE:ScheduleTimer("ClearNotify", CA.vars.notifyDelay)
		end
	end
end

function CA.OnGateChanged(eventCode, keepId, open)
	if ((CA.vars.outside == false) and (not IsInCampaign())) then
		do return end
	end
	
	local keepName = GetKeepName(keepId)
	
	if (timerIdExtra) then
		CA_ACE:CancelTimer(timerIdExtra)
	end
	
	if (not CA.vars.showGates) then
		do return end
	end
	
	if (open) then
		if (CA.vars.chatOutput) then
			d("|cB22222" .. keepName .. "|r |cE5E5E5is now Open!|r")
		end
		
		if ((not CA.vars.onlyChat) and (not IsInCampaign())) then
			CyrodiilAlertNotifyExtra:SetText("|cD6660C" .. keepName .. "|r |cE5E5E5is now Open!|r")

			timerIdExtra = CA_ACE:ScheduleTimer("ClearNotifyExtra", CA.vars.notifyDelay)
		end
	else
		if (CA.vars.chatOutput) then
			d("|cB22222" .. keepName .. "|r |cE5E5E5is now Closed!|r")
		end
		
		if ((not CA.vars.onlyChat) and (not IsInCampaign())) then
			CyrodiilAlertNotifyExtra:SetText("|cD6660C" .. keepName .. "|r |cE5E5E5is now Closed!|r")

			timerIdExtra = CA_ACE:ScheduleTimer("ClearNotifyExtra", CA.vars.notifyDelay)
		end
	end
end


function CA.OnDeposeEmperor(eventCode, campaignId, emperorName, emperorAlliance, abdication)
	if ((CA.vars.outside == false) and (not IsInCampaign())) then
		do return end
	end
	
	if (campaignId ~= GetPreferredCampaign()) then
		do return end
	end
	
	if (not CA.vars.showEmperors) then
		do return end
	end

	local allianceName = GetAllianceName(emperorAlliance)
	
--	if (timerIdExtra) then
--		CA_ACE:CancelTimer(timerIdExtra)
--	end
	
	if (CA.vars.chatOutput) then
		d("|cB22222" .. emperorName .."|r |cE5E5E5of |r |cD6660C" .. allianceName .. "|r |cE5E5E5has been deposed!|r")
	end
	
	if ((not CA.vars.onlyChat) and (not IsInCampaign())) then
		CyrodiilAlertNotifyExtra:SetText("|cB22222" .. emperorName .."|r |cE5E5E5of |r |cD6660C" ..  allianceName .. "|r |cE5E5E5has been deposed!|r")

		timerIdExtra = CA_ACE:ScheduleTimer("ClearNotifyExtra", CA.vars.notifyDelay)
	end

end

function CA.OnCoronateEmperor(eventCode, campaignId, emperorName, emperorAlliance)
	if ((CA.vars.outside == false) and (not IsInCampaign())) then
		do return end
	end
	
--	if (campaignId ~= GetPreferredCampaign()) then
--		do return end
--	end
	
	if (not CA.vars.showEmperors) then
		do return end
	end

	local allianceName = GetAllianceName(emperorAlliance)
	
	if (timerIdExtra) then
		CA_ACE:CancelTimer(timerIdExtra)
	end
	
	if (CA.vars.chatOutput) then
		d("|cB22222" .. emperorName .."|r |cE5E5E5of |r |cD6660C" .. allianceName .. "|r |cE5E5E5has been crowned Emperor!|r")
	end
	
	if ((not CA.vars.onlyChat) and (not IsInCampaign())) then
		CyrodiilAlertNotifyExtra:SetText("|cB22222" .. emperorName .."|r |cE5E5E5of |r |cD6660C" ..  allianceName .. "|r |cE5E5E5has been crowned Emperor!|r")

		timerIdExtra = CA_ACE:ScheduleTimer("ClearNotifyExtra", CA.vars.notifyDelay)
	end

end

function CA.OnArtifactControlState(eventCode, artifactName, keepId, playerName, playerAlliance, controlEvent, controlState, campaignId)
	if ((CA.vars.outside == false) and (not IsInCampaign())) then
		do return end
	end
	
	if (CA.vars.showScrolls == false) then
		do return end
	end

	local keepName = GetKeepName(keepId)
	local allianceName = GetAllianceName(playerAlliance)
	
	if (timerIdExtra) then
		CA_ACE:CancelTimer(timerIdExtra)
	end
	
	if (controlEvent == OBJECTIVE_CONTROL_EVENT_FLAG_TAKEN) then	
		if (CA.vars.chatOutput) then
			d("|cB22222" .. playerName .. "|r |cE5E5E5of|r |cD6660C" .. allianceName .. "|r |cE5E5E5has taken|r |cB22222" .. artifactName .. "|r")
		end
	
		if ((not CA.vars.onlyChat) and (not IsInCampaign())) then
			CyrodiilAlertNotifyExtra:SetText("|cB22222" .. playerName .. "|r |cE5E5E5of|r |cD6660C" .. allianceName .. "|r |cE5E5E5has taken|r |cB22222" .. artifactName .. "|r")

			timerIdExtra = CA_ACE:ScheduleTimer("ClearNotifyExtra", CA.vars.notifyDelay)
		end
			
	elseif (controlEvent == OBJECTIVE_CONTROL_EVENT_FLAG_DROPPED) then
		if (CA.vars.chatOutput) then
			d("|cB22222" .. playerName .. "|r |cE5E5E5of|r |cD6660C" .. allianceName .. "|r |cE5E5E5has dropped|r |cB22222" .. artifactName .. "|r")
		end
	
		if ((not CA.vars.onlyChat) and (not IsInCampaign())) then
			CyrodiilAlertNotifyExtra:SetText("|cB22222" .. playerName .. "|r |cE5E5E5of|r |cD6660C" .. allianceName .. "|r |cE5E5E5has dropped|r |cB22222" .. artifactName .. "|r")

			timerIdExtra = CA_ACE:ScheduleTimer("ClearNotifyExtra", CA.vars.notifyDelay)
		end
	
	elseif (controlEvent == OBJECTIVE_CONTROL_EVENT_CAPTURED) then
		if (CA.vars.chatOutput) then
			d("|cB22222" .. playerName .. "|r |cE5E5E5of|r |cD6660C" .. allianceName .. "|r |cE5E5E5has captured|r |cB22222" .. artifactName .. "|r |cE5E5E5at |r |cD6660C" .. keepName .. "|r")
		end
	
		if ((not CA.vars.onlyChat) and (not IsInCampaign())) then
			CyrodiilAlertNotifyExtra:SetText("|cB22222" .. playerName .. "|r |cE5E5E5of|r |cD6660C" .. allianceName .. "|r |cE5E5E5has captured|r |cB22222" .. artifactName .. "|r |cE5E5E5at |r |cD6660C" .. keepName .. "|r")

			timerIdExtra = CA_ACE:ScheduleTimer("ClearNotifyExtra", CA.vars.notifyDelay)
		end
	end
end

function CA.OnClaimKeep(eventCode, campaignId, keepId, guildName, playerName)
	if ((CA.vars.outside == false) and (not IsInCampaign())) then
		do return end
	end
	
	if (not CA.vars.showClaim) then
		do return end
	end

	local keepName = GetKeepName(keepId)
	local allianceName = GetAllianceName(emperorAlliance)
	
	if (timerIdExtra) then
		CA_ACE:CancelTimer(timerIdExtra)
	end
	
	if (CA.vars.chatOutput) then
		d("|cB22222" .. playerName .. "|r |cE5E5E5has claimed|r |cD6660C" .. keepName .. "|r |cE5E5E5for|r |cB22222" .. guildName .. "|r")
	end
	
	if (not CA.vars.onlyChat) then
		CyrodiilAlertNotifyExtra:SetText("|cB22222" .. playerName .. "|r |cE5E5E5has claimed|r |cD6660C" .. keepName .. "|r |cE5E5E5for|r |cB22222" .. guildName .. "|r")

		timerIdExtra = CA_ACE:ScheduleTimer("ClearNotifyExtra", CA.vars.notifyDelay)
	end

end

function CA_ACE:ClearNotify()
	CyrodiilAlertNotify:SetText("")
end

function CA_ACE:ClearNotifyTaken()
	CyrodiilAlertNotifyTaken:SetText("")
end

function CA_ACE:ClearNotifyExtra()
	CyrodiilAlertNotifyExtra:SetText("")
end

function CA.MoveStart()
    CyrodiilAlertBG:SetAlpha(0.5)
end 

function CA.MoveStop()
    CyrodiilAlertBG:SetAlpha(0)
    CA.vars.locx = math.floor(CyrodiilAlert:GetLeft())
    CA.vars.locy = math.floor(CyrodiilAlert:GetTop())
end 

function CA.SetTextAlign(newValue)
    CA.vars.textAlign = newValue
    local alignText = {}
    alignText["LEFT"]   = 0
    alignText["CENTER"] = 1
    alignText["RIGHT"]  = 2
    CyrodiilAlertNotify:SetHorizontalAlignment(alignText[newValue])
end

-- SLASH COMMAND FUNCTIONALITY
function CAslash(text )
	if (text == "show") then
		CyrodiilAlertBG:SetAlpha(0.5)
	end
	if (text == "hide") then
		CyrodiilAlertBG:SetAlpha(0)
	end
	if (text == "out") then
		if (CA.vars.outside) then
			CA.vars.outside = false
			d("Notifications outside of Cyrodiil turned OFF.")
		else
			CA.vars.outside = true
			d("Notifications outside of Cyrodiil turned ON.")
		end
	end
	if (text == "help") then
		d("Avalable slash commands: show, hide, help")
	end
	if (text == "test") then
		CyrodiilAlertNotifyTaken:SetText("|cD6660CAldmeri Dominion|r |cE5E5E5has captured|r |cD6660CFort Dragonclaw Farm|r")
		CyrodiilAlertNotify:SetText("|cD6660CKingscrest Keep|r |cE5E5E5is no longer under attack!|r")
		CyrodiilAlertNotifyExtra:SetText("|cB22222Beshaba Tanthul|r |cD6660Cof |r |cB22222Aldmeri Dominion|r |cE5E5E5has been crowned!|r")
--		CyrodiilAlertNotifyExtra:SetText("|cB22222Beshaba Tanthul|r |cE5E5E5of|r |cD6660CAldmeri Dominion|r |cE5E5E5has captured|r |cB22222Alma Ruma|r |cE5E5E5at|r |cD6660CKingscrest Keep|cB22222|r")
		timerId = CA_ACE:ScheduleTimer("ClearNotify", CA.vars.notifyDelay)
		timerIdTaken = CA_ACE:ScheduleTimer("ClearNotifyTaken", CA.vars.notifyDelay)
		timerIdExtra = CA_ACE:ScheduleTimer("ClearNotifyExtra", CA.vars.notifyDelay)
	end
end -- CAslash
 
SLASH_COMMANDS["/ca"] = CAslash